-----------------------------------------------------------------------------------------------
-- Client Lua Script for JoinARealm
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- JoinARealm Module Definition
-----------------------------------------------------------------------------------------------
local JoinARealm = {}
local RealmChannel = nil 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function JoinARealm:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function JoinARealm:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- JoinARealm OnLoad
-----------------------------------------------------------------------------------------------
function JoinARealm:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("JoinARealm.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- JoinARealm OnDocLoaded
-----------------------------------------------------------------------------------------------
function JoinARealm:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "JoinARealmForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end

		-- if the xmlDoc is no longer needed, you should set it to nil
		self.xmlDoc = nil
		self.wndMain:Show(false, true)
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		self.timer = ApolloTimer.Create(10.0, true, "OnTimer", self)
		Apollo.RegisterSlashCommand(string.lower(GameLib.GetRealmName()) .. "Help", "OnComHelp", self)
		Apollo.RegisterSlashCommand(string.lower(GameLib.GetRealmName()) .. "Data", "OnComRealmData", self)
		Apollo.RegisterSlashCommand(string.lower(GameLib.GetRealmName()), "OnComSend", self)
		Apollo.RegisterSlashCommand(string.lower(GameLib.GetRealmName()) .. "Version", "OnComGetVersion", self)
		Apollo.RegisterSlashCommand(string.lower(GameLib.GetRealmName()) .. "Credit", "OnComGetCredits", self)

		-- Do additional Addon initialization here
			-- Check if channel exists
		
		local bExists = false
		for id,channel in pairs(ChatSystemLib.GetChannels()) do
			if channel:GetName() == GameLib.GetRealmName() then
				bExists = true
				RealmChannel = channel
				break
			end
		end
		-- If it doesn't join the channel (and create it if it hasn't been yet)
		if not bExists then
			ChatSystemLib.Command("/chjoin " .. GameLib.GetRealmName())
		end
		if not RealmChannel then
			ChatSystemLib.PostOnChannel(ChatSystemLib.GetChannels()[2]:GetType(), "JoinARealm couldn't initialize instantly!", "[RealmChat]", GameLib.GetPlayerUnit(), 0)
			ChatSystemLib.PostOnChannel(ChatSystemLib.GetChannels()[2]:GetType(), "Please wait 10 seconds for it to initialize.", "[RealmChat]", GameLib.GetPlayerUnit(), 0)
		else
			ChatSystemLib.PostOnChannel(ChatSystemLib.GetChannels()[2]:GetType(), "JoinARealm is NOW initialized. Enjoy yourself ;)", "[RealmChat]", GameLib.GetPlayerUnit(), 0)
		end
	end
end

-----------------------------------------------------------------------------------------------
-- JoinARealm Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- *** INDEX ***
-- on SlashCommand "/<realm>Help"  -- This shows a list of all commands and a short info
-- on SlashCommand "/<realm>Version"  -- This shows the current version of the addon
-- on SlashCommand "/<realm>Credit"  -- This shows the author
-- on SlashCommand "/<realm>Data"  -- Shows debug information such as channel id and channel name
-- on SlashCommand "/<realm> <message>"  -- This sends a message to the <realm> channel

-- updates channel info
function JoinARealm:OnTimer()
	if not RealmChannel then
		local bExists = false
		for id,channel in pairs(ChatSystemLib.GetChannels()) do
			if channel:GetName() == GameLib.GetRealmName() then
				bExists = true
				RealmChannel = channel
				break
			end
		end
		-- If it doesn't join the channel (and create it if it hasn't been yet)
		if not bExists then
			ChatSystemLib.Command("/chjoin " .. GameLib.GetRealmName())
		end
		if not RealmChannel then
			ChatSystemLib.PostOnChannel(ChatSystemLib.GetChannels()[2]:GetType(), "JoinARealm couldn't initialize again!", "[RealmChat]", GameLib.GetPlayerUnit(), 0)
			ChatSystemLib.PostOnChannel(ChatSystemLib.GetChannels()[2]:GetType(), "Please wait another 10 seconds for it to initialize.", "[RealmChat]", GameLib.GetPlayerUnit(), 0)
		else
			ChatSystemLib.PostOnChannel(ChatSystemLib.GetChannels()[2]:GetType(), "JoinARealm is NOW initialized. Enjoy yourself ;)", "[RealmChat]", GameLib.GetPlayerUnit(), 0)
		end
	end
end

-- on SlashCommand "/<realm>Help"
function JoinARealm:OnComHelp()
	if RealmChannel then
		local realmName = string.lower(GameLib.GetRealmName())
		local lines = {"JoinARealm help", "The <realm> equals the name of your realm. E.g. <realm> equals jabbit on the Jabbit realm.", "Commands: (for your realm)",
		"/" .. realmName .. "Help    - Shows this help message", "/" .. realmName .. "Version    - Shows the JoinARealm version",
		"/" .. realmName .. "Credit    - Shows the author", "/" .. realmName .. "Data    - Shows debug information such as channel id and channel name",
		"/" .. realmName .. " <message>    - Sends a message to the realm channel of your realm"}
		local i = 1
		while i <= 8 do
			ChatSystemLib.PostOnChannel(ChatSystemLib.GetChannels()[2]:GetType(), lines[i] , "[RealmChat]", GameLib.GetPlayerUnit(), 0)
			i = i + 1
		end
	else
		ChatSystemLib.PostOnChannel(ChatSystemLib.GetChannels()[2]:GetType(), "Not initialized yet!! Impatient person, you.", "[RealmChat]", GameLib.GetPlayerUnit(), 0)
	end
end

-- on SlashCommand "/<realm>Version"
function JoinARealm:OnComGetVersion()
	if RealmChannel then
		ChatSystemLib.PostOnChannel(ChatSystemLib.GetChannels()[2]:GetType(), "Running Version 1.2.1", "[RealmChat]", GameLib.GetPlayerUnit(), 0)
	else
		ChatSystemLib.PostOnChannel(ChatSystemLib.GetChannels()[2]:GetType(), "Not initialized yet!! Impatient person, you.", "[RealmChat]", GameLib.GetPlayerUnit(), 0)
	end
end

-- on SlashCommand "/<realm>Credit"
function JoinARealm:OnComGetCredits()
	if RealmChannel then
		ChatSystemLib.PostOnChannel(ChatSystemLib.GetChannels()[2]:GetType(), "Addon and channel author: GreenyNeko (a.k.a. Swourcer Calisoul)", "[RealmChat]", GameLib.GetPlayerUnit(), 0)
	else
		ChatSystemLib.PostOnChannel(ChatSystemLib.GetChannels()[2]:GetType(), "Not initialized yet!! Impatient person, you.", "[RealmChat]", GameLib.GetPlayerUnit(), 0)
	end
end
		
-- on SlashCommand "/<realm>Data"
function JoinARealm:OnComRealmData()
	if RealmChannel then
		ChatSystemLib.PostOnChannel(ChatSystemLib.GetChannels()[2]:GetType(), "Realm Channel ID: " .. RealmChannel:GetType(), "[RealmChat]", GameLib.GetPlayerUnit(), 0)
		ChatSystemLib.PostOnChannel(ChatSystemLib.GetChannels()[2]:GetType(), "Realm Channel Name: " .. RealmChannel:GetName(), "[RealmChat]", GameLib.GetPlayerUnit(), 0)
	else
		ChatSystemLib.PostOnChannel(ChatSystemLib.GetChannels()[2]:GetType(), "Not initialized yet!! Impatient person, you.", "[RealmChat]", GameLib.GetPlayerUnit(), 0)
	end
end

-- on SlashCommand "/<realm>"
function JoinARealm:OnComSend(sCommand, sMessage)
	if RealmChannel then
		RealmChannel:Send(sMessage)
	else
		ChatSystemLib.PostOnChannel(ChatSystemLib.GetChannels()[2]:GetType(), "Not initialized yet!! Impatient person, you.", "[RealmChat]", GameLib.GetPlayerUnit(), 0)
	end
end

-----------------------------------------------------------------------------------------------
-- JoinARealm Instance
-----------------------------------------------------------------------------------------------
local JoinARealmInst = JoinARealm:new()
JoinARealmInst:Init()
